#include <iostream>
#include <fstream>
#include <random>
#include <vector>
#include <set>
#include <cassert>
#include <unordered_set>
#include <algorithm>
#include <climits>

// rand() nepouzivame, napr. protoze neni definovane, jak velka cisla muze vracet
#define rand(x) ("Misto rand() prosim pouzivej funkce rand* definovane zde", +)
#define ll long long
using namespace std;

// Generator nahodnych cisel
std::mt19937_64 rng;

ll randRange(ll from, ll to) {
    to = max(to, from);
    // Vrati nahodny long long v intervalu [from, to] (tj. vcetne `to`).
    std::uniform_int_distribution<ll> dist(from, to);
    return dist(rng);
}

double randDouble(double from, double to) {
    // Vrati nahodny double v intervalu [from, to), (tj. bez `to`).
    std::uniform_real_distribution<double> dist(from, to);
    return dist(rng);
}

ll randAlmost(ll to, double min_part = 0.9) {
    // Vrati nahodny long long ktery je v nejakem smyslu "skoro tak velky" jako `to`,
    // konkretne vrati cislo v intervalu [floor(value*min_part), value].
    // Uzitecne napr. pro generovani velikosti problemu
    if(to == 0) return 0;
    assert(to >= 1);
    return randRange(min_part * to, to);
}

bool randBool(double p = 0.5) {
    // Vrati true s pravdepodobnosti p a false s pravdepodobnosti 1-p
    return randDouble(0, 1) <= p;
}

vector<ll> randDifferentNumbers(ll from, ll to, int k) {
    // Vrati k ruznych nahodnych cisel z intervalu [from, to]
    // Implementace podle: https://www.gormanalysis.com/blog/random-numbers-in-cpp/

    unordered_set<ll> samples;
    ll n = to - from + 1;
    assert(n >= k);

    for (ll r = n - k; r < n; ++r) {
        ll v = uniform_int_distribution<ll>(from, from + r)(rng);
        if (!samples.insert(v).second) samples.insert(from + r);
    }

    vector<ll> result(samples.begin(), samples.end());
    shuffle(result.begin(), result.end(), rng);

    return result;
}


const int MIN_N = 1;
const int MAX_N[3] = {100*1000, 100*1000, 100*1000};
const int MAX_M[3] = {300*1000, 300*1000, 300*1000};
const int MIN_VALUE = 1;
const int MAX_VALUE = 1e6;
const int T = 20;



struct Problem {
    // Reprezentuje jeden problem (test).
    vector<pair<pair<int,int>, int> > edges;
    vector<int> a;

    Problem() {}

    void check(int diff) {
        // Tato metoda neni nutna, ale je uzitecna jako zakladni kontrola:
        // zkontroluje, jestli problem odpovida limitum stanovenym v zadani
        int n = a.size();
        int m = edges.size();
        assert(1 <= n && n <= MAX_N[diff]);
        assert(0 <= m && m <= MAX_M[diff]);

        for(int i = 0; i < n; ++i){
            assert(1 <= a[i] && a[i] <= MAX_VALUE);
        }

        for(int i = 0; i < m; ++i){
            int u = edges[i].first.first;
            int v = edges[i].first.second;
            int w = edges[i].second;
            assert(1 <= w && w <= MAX_VALUE);
            assert(1 <= u && u <= n);
            assert(1 <= v && v <= n);
        }

        if(diff+1 == 1){
            assert(m == n-1);
            for(int i = 0; i < m; ++i){
                int u = edges[i].first.first;
                int v = edges[i].first.second;
                assert(u == i+1 && v == i+2);
            }
        }

        if(diff+1 == 2){
            int mn = INT_MAX;
            for(int i = 0; i < n; ++i){
                mn = min(mn, a[i]);
            }
            for(auto e : edges){
                assert(e.second < mn);
            }
        }

    }

    void print(int diff, int it) {
        // Vypise problem tak, jak to specifikuje sekce "Tvar vstupu"
        ofstream outstream;
        char type = '0'+diff+1;
        char subtype = 'a' + it;

        outstream.open("data/" + string(1,type) + string(1,subtype) + ".in");
        outstream << a.size() << " " << edges.size() << endl;
        for(int i = 0; i < a.size(); ++i){
            outstream << a[i] << " \n"[i==a.size()-1];
        }
        for(int i = 0; i < edges.size(); ++i){
            outstream << edges[i].first.first << " " << edges[i].first.second << " " << edges[i].second << endl;
        }
        outstream.close();
    }
};

Problem genCesta(int n, int max_val){
    Problem problem; 
    problem.a.resize(n);

    for(int i = 0; i < n; ++i){
        problem.a[i] = randRange(1, max_val);
    }

    for(int i = 0; i < n-1; ++i){
        problem.edges.push_back({{i+1, i+2}, randRange(1, max_val)});
    }

    return problem;
}

Problem genRandom(int n, int m, int max_val){
    Problem problem;

    for(int i = 0; i < n; ++i){
        problem.a.push_back(randRange(1, max_val));
    }

    set<pair<int,int> > hrany;
    while(hrany.size() < m){
        int u = randRange(1, n);
        int v = randRange(1, n);
        if(u < v && hrany.count({u,v}) == 0){
            hrany.insert({u,v});
        }
    }

    for(auto hrana : hrany){
        int u = hrana.first;
        int v = hrana.second;
        if(randRange(0,1) == 0) swap(u,v);

        problem.edges.push_back({{u,v}, randRange(1, max_val)});
    }


    return problem;
}

Problem genRandom(int n, int m, int max_val_edges, int max_val_nodes){
    Problem problem;

    for(int i = 0; i < n; ++i){
        problem.a.push_back(randRange(max_val_edges + 1, max_val_nodes));
    }

    set<pair<int,int> > hrany;
    while(hrany.size() < m){
        int u = randRange(1, n);
        int v = randRange(1, n);
        if(u < v && hrany.count({u,v}) == 0){
            hrany.insert({u,v});
        }
    }

    for(auto hrana : hrany){
        int u = hrana.first;
        int v = hrana.second;
        if(randRange(0,1) == 0) swap(u,v);

        problem.edges.push_back({{u,v}, randRange(1, max_val_edges)});
    }


    return problem;
}

Problem genSample1(){
    Problem problem;
    problem.a = {6, 5, 2, 3, 3, 3};
    problem.edges.push_back({{1, 2}, 1});
    problem.edges.push_back({{2, 3}, 8});
    problem.edges.push_back({{3, 4}, 7});
    problem.edges.push_back({{4, 5}, 1});
    problem.edges.push_back({{5, 6}, 1});

    return problem;
}

Problem genSample2(){
    Problem problem;
    problem.a = {10, 10, 10, 10, 10, 10};

    problem.edges.push_back({{1, 2}, 1});
    problem.edges.push_back({{2, 3}, 1});
    problem.edges.push_back({{1, 3}, 1});
    problem.edges.push_back({{4, 5}, 1});
    problem.edges.push_back({{5, 6}, 1});
    problem.edges.push_back({{4, 6}, 1});
    problem.edges.push_back({{1, 4}, 10});
    return problem;
}

Problem genTrivial(){
    Problem problem;
    problem.a = {42, 42};
    return problem;
}

Problem genOnenode(){
    Problem problem;
    problem.a = {42};
    return problem;
}

vector<Problem> genAll(int diff) {
    int max_n = MAX_N[diff];
    int max_m = MAX_M[diff];
    vector<Problem> problems;

    if(diff+1 == 1){//cesta
        for (int i = 0; i < 2; i++) {
            problems.push_back(genCesta(randAlmost(max_n), 3));
            problems.push_back(genCesta(randAlmost(max_n), 10));
            problems.push_back(genCesta(randAlmost(max_n), 1000));
            problems.push_back(genCesta(randAlmost(max_n), MAX_VALUE));
        }

        problems.push_back(genSample1());
        problems.push_back(genOnenode());
        
    }

    if(diff+1 == 2){// jedna elektrarna
        for (int i = 0; i < 2; i++) {
            problems.push_back(genRandom(randAlmost(max_n), randAlmost(max_m), 3, 10));
            problems.push_back(genRandom(randAlmost(max_n), randAlmost(max_m), 10, 100));
            problems.push_back(genRandom(randAlmost(max_n), randAlmost(max_m), 1000, 100000));
            problems.push_back(genRandom(randAlmost(max_n), randAlmost(max_m), 100000, MAX_VALUE));
        }
        problems.push_back(genOnenode());
    }

    if(diff+1 == 3){
        for (int i = 0; i < 2; i++) {
            problems.push_back(genRandom(randAlmost(max_n), randAlmost(max_m), 3));
            problems.push_back(genRandom(randAlmost(max_n), randAlmost(max_m), 10));
            problems.push_back(genRandom(randAlmost(max_n), randAlmost(max_m), 1000));
            problems.push_back(genRandom(randAlmost(max_n), randAlmost(max_m), MAX_VALUE));
        }
        problems.push_back(genTrivial());
        problems.push_back(genSample1());
        problems.push_back(genSample2());
        problems.push_back(genOnenode());
    }

    return problems;
}

int main(int argc, char* argv[]) {
    if (argc != 3) {
        cout << "pouziti: ./gen diff seed" << endl;
        cout << "    diff : 1 pro lehkou variantu, 2 pro tezkou" << endl;
        cout << "    seed : hexadecimalni seed pro nahodny generator" << endl;
        exit(1);
    }

    // diff iteruje pres typy datasetu
    // POZOR odcitam jednicku
    int diff = atoi(argv[1]) - 1;


    // Tento kouzelny radek nacte hexadecimalni seed (napr. 1C -> 28)
    int seed = strtoull(argv[2], NULL, 16) & 0x7fffffff;
    rng.seed(seed);

    vector<Problem> problems = genAll(diff);

    shuffle(problems.begin(), problems.end(), rng);
    assert(problems.size() <= T);

    for (int i = 0; i < problems.size(); ++i) {
        problems[i].check(diff);
        problems[i].print(diff, i);
    }
}
